<?php

/**
 * @file
 * Replaces the core Filter module administration pages.
 */

/**
 * Implements hook_form_FORM_ID_alter().
 *
 * Adds editor support to the text format add/edit form.
 */
function editor_form_filter_admin_format_form_alter(&$form, &$form_state, $form_id) {
  $format = isset($form_state['format']) ? $form_state['format'] : NULL;

  if ($format === NULL) {
    $format = $form_state['build_info']['args'][0];
    editor_format_ensure_additional_properties($format);
    $form_state['format'] = $format;
  }

  $form['#attached']['library'][] = array('filter', 'filter.admin');

  // Assign weights to existing form items.
  $form['name']['#weight'] = -30;
  $form['format']['#weight'] = -20;
  $form['roles']['#weight'] = -10;

  $editor_options = editor_list_options();
  $form['editor'] = array(
    // Position the editor selection before the filter settings (weight of 0),
    // but after the filter label and name (weight of -20).
    '#weight' => -9,
  );
  $form['editor']['editor'] = array(
    '#type' => 'select',
    '#title' => t('Text editor'),
    '#options' => $editor_options,
    '#empty_option' => t('None'),
    '#default_value' => isset($format->editor) ? $format->editor : '',
    '#ajax' => array(
      'trigger_as' => array('name' => 'editor_configure'),
      'callback' => 'editor_admin_format_editor_ajax',
      'wrapper' => 'editor-settings-wrapper',
    ),
    '#weight' => -10,
  );
  $form['editor']['configure'] = array(
    '#type' => 'submit',
    '#name' => 'editor_configure',
    '#value' => t('Configure'),
    '#limit_validation_errors' => array(array('editor')),
    '#submit' => array('editor_admin_format_editor_configure'),
    '#ajax' => array(
      'callback' => 'editor_admin_format_editor_ajax',
      'wrapper' => 'editor-settings-wrapper',
    ),
    '#weight' => -10,
    '#attributes' => array('class' => array('js-hide')),
  );

  // If there aren't any options (other than "None"), disable the select list.
  if (empty($editor_options)) {
    $form['editor']['editor']['#disabled'] = TRUE;
    $form['editor']['editor']['#description'] = t('This option is disabled because no modules that provide a text editor are currently enabled.');
  }

  $form['editor']['settings'] = array(
    '#tree' => TRUE,
    '#weight' => -8,
    '#type' => 'container',
    '#id' => 'editor-settings-wrapper',
    '#attached' => array(
      'library' => array(
        array('editor', 'editor.admin'),
      ),
    ),
  );

  $editor = isset($format->editor) ? $format->editor : '';

  // Add editor-specific validation and submit handlers.
  if ($editor) {
    $editors = editor_get_editors();
    $plugin = $editors[$editor];
    $form_state['editor_plugin'] = $plugin;

    // Load the associated editor callbacks file, if any.
    if (!empty($plugin['file'])) {
      $filepath = $plugin['file'];
      $extension = substr($filepath, strrpos($filepath, '.') + 1);
      $filepath = substr($filepath, 0, strrpos($filepath, '.'));
      form_load_include($form_state, $extension, $plugin['module'], $filepath);
    }

    // Populate editor defaults.
    if (!empty($plugin['default settings'])) {
      $format->editor_settings += $plugin['default settings'];
    }

    // Save default settings.
    $form_state['format'] = $format;

    // Generate editor settings.
    if (!empty($plugin['settings callback'])) {
      $function = $plugin['settings callback'];
      $form['editor']['settings']['subform'] = array();
      $form['editor']['settings']['subform'] = $function($form['editor']['settings']['subform'], $form_state);
      $form['editor']['settings']['subform']['#parents'] = array('editor', 'settings');
    }
  }

  $form['#validate'][] = 'editor_admin_format_editor_validate';
  array_unshift($form['#submit'], 'editor_admin_format_editor_submit');
}

/**
 * Button submit handler for filter_format_form()'s 'editor_configure' button.
 */
function editor_admin_format_editor_configure($form, &$form_state) {
  $format = $form_state['format'];
  $editor_value = $form_state['values']['editor']['editor'];
  if ($editor_value !== NULL) {
    if ($editor_value === '') {
      $form_state['editor_plugin'] = NULL;
    }
    elseif (empty($format) || $editor_value !== $format->editor) {
      $format = $form_state['build_info']['args'][0];
      $format->editor = $editor_value;
      $form_state['format'] = $format;
    }
  }

  $form_state['rebuild'] = TRUE;
}

/**
 * AJAX callback handler for filter_format_form().
 */
function editor_admin_format_editor_ajax($form, $form_state) {
  return $form['editor']['settings'];
}

/**
 * Additional validate handler for filter_format_form().
 */
function editor_admin_format_editor_validate($form, &$form_state) {
  $editor_set = $form_state['values']['editor']['editor'] !== "";
  $subform_array_exists = (!empty($form['editor']['settings']['subform']) && is_array($form['editor']['settings']['subform']));
  $editor_plugin = !empty($form_state['editor_plugin']) ? $form_state['editor_plugin'] : NULL;
  if ($editor_set && $subform_array_exists && $editor_plugin) {
    $editor_plugin['validate callback']($form['editor']['settings']['subform'], $form_state);
  }

  // This validate handler is not applicable when using the 'Configure' button.
  if ($form_state['triggering_element']['#name'] === 'editor_configure') {
    return;
  }

  // When using this form with JavaScript disabled in the browser, the
  // 'Configure' button won't be clicked automatically. So, when the user has
  // selected a text editor and has then clicked 'Save configuration', we should
  // point out that the user must still configure the text editor.
  if ($form_state['values']['editor']['editor'] !== '' && !$form_state['values']['editor']) {
    form_set_error('editor][editor', t('You must configure the selected text editor.'));
  }
}

/**
 * Additional submit handler for filter_format_form().
 */
function editor_admin_format_editor_submit($form, &$form_state) {
  $editor_set = $form_state['values']['editor']['editor'] !== "";
  $subform_array_exists = (!empty($form['editor']['settings']['subform']) && is_array($form['editor']['settings']['subform']));
  $editor_plugin = !empty($form_state['editor_plugin']) ? $form_state['editor_plugin'] : NULL;
  if ($editor_set && $subform_array_exists && $editor_plugin) {
    $editor_plugin['submit callback']($form['editor']['settings']['subform'], $form_state);
  }

  $editor = '';
  $editor_settings = array();
  $image_upload_settings = array();

  // Create a new editor or update the existing editor.
  if (!empty($form_state['format'])) {
    $format = $form_state['format'];
    $editor = $format->editor;
    $image_upload_settings = $format->image_upload;
    if (!empty($form_state['values']['editor_settings'])) {
      $editor_settings = $form_state['values']['editor_settings'];
    }
  }

  // All $form_state['values'] are added to the $format object and saved in
  // filter_admin_format_form_submit(). Add the editor, editor_settings and
  // image upload settings as form state values so that they get copied. This
  // also overwrites the unwanted editor information stored in
  // $form_state['values']['editor'].
  $form_state['values']['editor'] = $editor;
  $form_state['values']['editor_settings'] = $editor_settings;
  $form_state['values']['image_upload'] = $image_upload_settings;
}

/**
 * Subform constructor to configure the text editor's image upload settings.
 *
 * Each text editor plugin that is configured to offer the ability to insert
 * images and uses EditorImageDialog for that, should use this form to update
 * the text editor's configuration so that EditorImageDialog knows whether it
 * should allow the user to upload images.
 *
 * @param object $format
 *   The text format that is being edited.
 *
 * @return array
 *   The image upload settings form.
 *
 * @ingroup forms
 */
function editor_image_upload_settings_form($format) {
  // Defaults.
  $image_upload = editor_get_image_upload_settings($format);
  $image_upload += array(
    'status' => FALSE,
    'scheme' => file_default_scheme(),
    'directory' => 'inline-images',
    'max_size' => '',
    'max_dimensions' => array('width' => '', 'height' => ''),
  );

  $form['status'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enable image uploads'),
    '#default_value' => $image_upload['status'],
    '#attributes' => array(
      'data-editor-image-upload' => 'status',
    ),
  );

  $show_if_image_uploads_enabled = array(
    'visible' => array(
      ':input[data-editor-image-upload="status"]' => array('checked' => TRUE),
    ),
  );

  // Any visible, writable wrapper can potentially be used for uploads,
  // including a remote file system that integrates with a CDN.
  $stream_wrappers = file_get_stream_wrappers(STREAM_WRAPPERS_WRITE_VISIBLE);

  foreach ($stream_wrappers as $scheme => $info) {
    $options[$scheme] = check_plain($info['description']);
  }

  if (!empty($options)) {
    $form['scheme'] = array(
      '#type' => 'radios',
      '#title' => t('File storage'),
      '#default_value' => $image_upload['scheme'],
      '#options' => $options,
      '#states' => $show_if_image_uploads_enabled,
      '#access' => count($options) > 1,
    );
  }

  // Set data- attributes with human-readable names for all possible stream
  // wrappers, so that drupal.ckeditor.drupalimage.admin's summary rendering
  // can use that.
  foreach ($stream_wrappers as $scheme => $info) {
    $form['scheme'][$scheme]['#attributes']['data-label'] = t('Storage: @name', array('@name' => $info['name']));
  }

  $form['directory'] = array(
    '#type' => 'textfield',
    '#default_value' => $image_upload['directory'],
    '#title' => t('Upload directory'),
    '#description' => t("A directory relative to Drupal's files directory where uploaded images will be stored."),
    '#states' => $show_if_image_uploads_enabled,
  );

  $default_max_size = format_size(file_upload_max_size());
  $form['max_size'] = array(
    '#type' => 'textfield',
    '#default_value' => $image_upload['max_size'],
    '#title' => t('Maximum file size'),
    '#description' => t('If this is left empty, then the file size will be limited by the PHP maximum upload size of @size.', array('@size' => $default_max_size)),
    '#maxlength' => 20,
    '#size' => 10,
    '#placeholder' => $default_max_size,
    '#states' => $show_if_image_uploads_enabled,
  );

  $form['max_dimensions'] = array(
    '#type' => 'item',
    '#title' => t('Maximum dimensions'),
    '#description' => t('Images larger than these dimensions will be scaled down.'),
    '#states' => $show_if_image_uploads_enabled,
  );
  $form['max_dimensions']['width'] = array(
    '#title' => t('Width'),
    '#title_display' => 'invisible',
    '#type' => module_exists('elements') ? 'numberfield' : 'textfield',
    '#default_value' => (empty($image_upload['max_dimensions']['width'])) ? '' : $image_upload['max_dimensions']['width'],
    '#size' => 8,
    '#maxlength' => 8,
    '#min' => 1,
    '#max' => 99999,
    '#placeholder' => t('width'),
    '#field_suffix' => ' x ',
    '#states' => $show_if_image_uploads_enabled,
    '#prefix' => '<div class="form--inline clearfix">',
  );
  $form['max_dimensions']['height'] = array(
    '#title' => t('Height'),
    '#title_display' => 'invisible',
    '#type' => module_exists('elements') ? 'numberfield' : 'textfield',
    '#default_value' => (empty($image_upload['max_dimensions']['height'])) ? '' : $image_upload['max_dimensions']['height'],
    '#size' => 8,
    '#maxlength' => 8,
    '#min' => 1,
    '#max' => 99999,
    '#placeholder' => t('height'),
    '#field_suffix' => t('pixels'),
    '#states' => $show_if_image_uploads_enabled,
    '#suffix' => '</div>',
  );

  return $form;
}
