<?php

/**
 * @file
 * Editor CKEditor hooks implemented by the Editor CKEditor module.
 */

/**
 * Implements hook_editor_ckeditor_plugins().
 */
function editor_ckeditor_editor_ckeditor_plugins() {
  $path = drupal_get_path('module', 'editor_ckeditor');
  $image_prefix = $path . '/images/buttons/';

  $buttons = array(
    // "basicstyles" plugin.
    'Bold' => array(
      'label' => t('Bold'),
      'required_html' => array(array('tags' => array('strong'))),
    ),
    'Italic' => array(
      'label' => t('Italic'),
      'required_html' => array(array('tags' => array('em'))),
    ),
    'Underline' => array(
      'label' => t('Underline'),
      // A class is used on spans for underline.
      'required_html' => array(array('tags' => array('span'))),
    ),
    'Strike' => array(
      'label' => t('Strike-through'),
      'required_html' => array(array('tags' => array('del'))),
    ),
    'Superscript' => array(
      'label' => t('Superscript'),
      'required_html' => array(array('tags' => array('sup'))),
    ),
    'Subscript' => array(
      'label' => t('Subscript'),
      'required_html' => array(array('tags' => array('sub'))),
    ),
    // "removeformat" plugin.
    'RemoveFormat' => array(
      'label' => t('Remove format'),
    ),
    // "justify" plugin.
    'JustifyLeft' => array(
      'label' => t('Align left'),
      'required_html' => array(array('tags' => array('p'), 'classes' => array('text-align-left'))),
    ),
    'JustifyCenter' => array(
      'label' => t('Align center'),
      'required_html' => array(array('tags' => array('p'), 'classes' => array('text-align-center'))),
    ),
    'JustifyRight' => array(
      'label' => t('Align right'),
      'required_html' => array(array('tags' => array('p'), 'classes' => array('text-align-right'))),
    ),
    'JustifyBlock' => array(
      'label' => t('Justify'),
      'required_html' => array(array('tags' => array('p'), 'classes' => array('text-align-justify'))),
    ),
    // "list" plugin.
    'BulletedList' => array(
      'label' => t('Bullet list'),
      'image_rtl' => $image_prefix . '/bulletedlist-rtl.png',
      'required_html' => array(array('tags' => array('ul', 'li'))),
    ),
    'NumberedList' => array(
      'label' => t('Numbered list'),
      'image_rtl' => $image_prefix . '/numberedlist-rtl.png',
      'required_html' => array(array('tags' => array('ol', 'li'))),
    ),
    // "indent" plugin.
    'Outdent' => array(
      'label' => t('Outdent'),
      'image_rtl' => $image_prefix . '/outdent-rtl.png',
      'required_html' => array(array('tags' => array('ul', 'ol'))),
    ),
    'Indent' => array(
      'label' => t('Indent'),
      'image_rtl' => $image_prefix . '/indent-rtl.png',
      'required_html' => array(array('tags' => array('ul', 'ol'))),
    ),
    // "undo" plugin.
    'Undo' => array(
      'label' => t('Undo'),
      'image_rtl' => $image_prefix . '/undo-rtl.png',
    ),
    'Redo' => array(
      'label' => t('Redo'),
      'image_rtl' => $image_prefix . '/redo-rtl.png',
    ),
    // "blockquote" plugin.
    'Blockquote' => array(
      'label' => t('Blockquote'),
      'required_html' => array(array('tags' => array('blockquote'))),
    ),
    // "horizontalrule" plugin
    'HorizontalRule' => array(
      'label' => t('Horizontal rule'),
      'required_html' => array(array('tags' => array('hr'))),
    ),
    // "clipboard" plugin.
    'Cut' => array(
      'label' => t('Cut'),
    ),
    'Copy' => array(
      'label' => t('Copy'),
    ),
    'Paste' => array(
      'label' => t('Paste'),
    ),
    // "pastetext" plugin.
    'PasteText' => array(
      'label' => t('Paste Text'),
      'image_rtl' => $image_prefix . '/pastetext-rtl.png',
    ),
    // "pastefromword" plugin.
    'PasteFromWord' => array(
      'label' => t('Paste from Word'),
      'image_rtl' => $image_prefix . '/pastefromword-rtl.png',
    ),
    // "specialchar" plugin.
    'SpecialChar' => array(
      'label' => t('Character map'),
    ),
    'Format' => array(
      'label' => t('HTML block format'),
      'image_alternative' => '<span class="ckeditor-button-dropdown">' . t('Format') . '<span class="ckeditor-button-arrow"></span></span>',
    ),
    // "table" plugin.
    'Table' => array(
      'label' => t('Table'),
      'required_html' => array(array('tags' => array('table', 'thead', 'tbody', 'tr', 'td', 'th'))),
    ),
    // "showblocks" plugin.
    'ShowBlocks' => array(
      'label' => t('Show blocks'),
      'image_rtl' => $image_prefix . '/showblocks-rtl.png',
    ),
    // "sourcearea" plugin.
    'Source' => array(
      'label' => t('Source code'),
    ),
    // "maximize" plugin.
    'Maximize' => array(
      'label' => t('Maximize'),
    ),
    // No plugin, separator "button" for toolbar builder UI use only.
    '-' => array(
      'label' => t('Separator'),
      'image_alternative' => '<span class="ckeditor-separator" title="' . t('Button separator') . '" aria-label="' . t('Button separator') . '">&nbsp;</span>',
      'attributes' => array('class' => array('ckeditor-button-separator')),
      'multiple' => TRUE,
    ),
  );

  // Populate image locations that match button names.
  foreach ($buttons as $button_name => &$button) {
    if (!isset($button['image_alternative']) && !isset($button['image'])) {
      // Because button names are ASCII text, drupal_strtolower() is not needed.
      $button['image'] = $image_prefix . strtolower($button_name) . '.png';
    }
  }

  // List all the basic plugin buttons as an "internal" plugin.
  $plugins['internal'] = array(
    'label' => t('CKEditor core'),
    'file' => $path . '/plugins/editor_ckeditor.internal.inc',
    'internal' => TRUE,
    'contextual' => TRUE,
    'enabled callback' => 'editor_ckeditor_internal_plugin_check',
    'buttons' => $buttons,
    'config callback' => 'editor_ckeditor_internal_config',
  );

  $plugins['stylescombo'] = array(
    'label' => t('Styles dropdown'),
    'file' => $path . '/plugins/editor_ckeditor.stylescombo.inc',
    'internal' => TRUE,
    'buttons' => array(
      'Styles' => array(
        'label' => t('Font style'),
        'image_alternative' => '<span class="ckeditor-button-dropdown">' . t('Styles') . '<span class="ckeditor-button-arrow"></span></span>',
      ),
    ),
    'supports_configuration' => TRUE,
    'config callback' => 'editor_ckeditor_stylescombo_config',
    'settings form' => 'editor_ckeditor_stylescombo_settings_form',
  );

  $plugins['language'] = array(
    'label' => t('Language'),
    'file' => $path . '/plugins/editor_ckeditor.language.inc',
    'internal' => TRUE,
    'buttons' => array(
      'Language' => array(
        'label' => t('Language'),
        'required_html' => array(
          array(
            'tags' => array('span'),
            'attributes' => array('lang', 'dir'),
          ),
        ),
        'image' => $image_prefix . '/language.png',
      ),
    ),
    'libraries' => array(array('editor_ckeditor', 'editor_ckeditor.plugins.language')),
    'css' => array(drupal_get_path('module', 'editor_ckeditor') . '/css/plugins/language/editor_ckeditor.language.css'),
    'supports_configuration' => TRUE,
    'config callback' => 'editor_ckeditor_language_config',
    'settings form' => 'editor_ckeditor_language_settings_form',
  );

  // The drupalimage plugin replaces normal image functionality.
  $plugins['drupalimage'] = array(
    'label' => t('Image'),
    'file' => $path . '/plugins/editor_ckeditor.drupalimage.inc',
    'plugin' => drupal_get_path('module', 'editor_ckeditor') . '/js/plugins/drupalimage/plugin.js',
    'buttons' => array(
      'DrupalImage' => array(
        'label' => t('Image'),
        'required_html' => array(
          array(
            'tags' => array('img'),
            'attributes' => array('src', 'alt', 'data-entity-type', 'data-entity-id', 'data-align', 'data-caption'),
          ),
        ),
        'optional_html' => array(
          array(
            'attributes' => array('width', 'height', 'data-entity-uuid'),
          ),
        ),
        'image' => drupal_get_path('module', 'editor_ckeditor') . '/js/plugins/drupalimage/icons/drupalimage.png',
      ),
    ),
    'libraries' => array(array('system', 'drupal.ajax')),
    'supports_configuration' => TRUE,
    'config callback' => 'editor_ckeditor_drupalimage_config',
    'settings form' => 'editor_ckeditor_drupalimage_settings_form',
  );

  // The drupalcaption plugin provides consistent behaviors for image captions.
  $plugins['drupalimagecaption'] = array(
    'label' => t('Caption'),
    'file' => $path . '/plugins/editor_ckeditor.drupalimagecaption.inc',
    'plugin' => drupal_get_path('module', 'editor_ckeditor') . '/js/plugins/drupalimagecaption/plugin.js',
    'contextual' => TRUE,
    'internal' => FALSE,
    'dependencies' => array(),
    'enabled callback' => 'editor_ckeditor_image_caption_plugin_check',
    'required_html' => array(
      array(
        'tags' => array('img'),
        'attributes' => array('data-align', 'data-caption'),
      ),
    ),
    'libraries' => array(array('editor_ckeditor', 'editor_ckeditor.plugins.drupalimagecaption')),
    'css' => array(drupal_get_path('module', 'editor_ckeditor') . '/css/plugins/drupalimagecaption/editor_ckeditor.drupalimagecaption.css'),
    'config callback' => 'editor_ckeditor_drupalimagecaption_config',
  );

  // The drupallink plugin replaces normal link functionality.
  $plugins['drupallink'] = array(
    'label' => t('Link'),
    'file' => $path . '/plugins/editor_ckeditor.drupallink.inc',
    'plugin' => drupal_get_path('module', 'editor_ckeditor') . '/js/plugins/drupallink/plugin.js',
    'buttons' => array(
      'DrupalLink' => array(
        'label' => t('Link'),
        'required_html' => array(array(
          'tags' => array('a'),
          'attributes' => array('href'),
        )),
        'image' => drupal_get_path('module', 'editor_ckeditor') . '/js/plugins/drupallink/icons/drupallink.png',
      ),
      'DrupalUnlink' => array(
        'label' => t('Unlink'),
        'image' => drupal_get_path('module', 'editor_ckeditor') . '/js/plugins/drupallink/icons/drupalunlink.png',
      ),
    ),
    'libraries' => array(array('system', 'drupal.ajax')),
    'config callback' => 'editor_ckeditor_drupallink_config',
  );

  return $plugins;
}

/**
 * Implements hook_editor_ckeditor_css_alter().
 */
function editor_ckeditor_editor_ckeditor_css_alter(array &$css, $format) {
  $filters = filter_list_format($format->format);

  // Add the editor caption CSS if the text format associated with this text
  // editor uses the editor_caption filter. This is used by the included
  // CKEditor DrupalImageCaption plugin.
  if (isset($filters['editor_caption']) && $filters['editor_caption']->status) {
    $css[] = drupal_get_path('module', 'editor') . '/css/filter/filter.caption.css';
  }
}
