<?php

/**
 * Admin menu callbacks and theme functions for Editor CKEditor module.
 */

/**
 * Editor settings callback; Provide options for Editor CKEditor module.
 */
function editor_ckeditor_settings_form(&$form, $form_state) {
  $format = $form_state['format'];
  $settings = $format->editor_settings;

  $ckeditor_settings_toolbar = array(
    '#theme' => 'editor_ckeditor_settings_toolbar',
    '#format' => $format,
    '#plugins' => editor_ckeditor_get_buttons(),
  );

  $form['toolbar'] = array(
    '#type' => 'container',
    '#attached' => array(
      'library' => array(
        array('editor_ckeditor', 'editor_ckeditor.admin'),
      ),
      'js' => array(
        array('data' => array('ckeditor' => array('toolbarAdmin' => drupal_render($ckeditor_settings_toolbar))), 'type' => 'setting'),
      ),
    ),
    '#attributes' => array('class' => array('ckeditor-toolbar-configuration')),
  );

  $form['toolbar']['button_groups'] = array(
    '#type' => 'textarea',
    '#title' => t('Toolbar buttons'),
    '#default_value' => json_encode($settings['toolbar']),
    '#attributes' => array('class' => array('ckeditor-toolbar-textarea')),
  );

  // CKEditor plugin settings, if any.
  $form['plugin_settings_title'] = array(
    '#type' => 'item',
    '#title' => t('CKEditor plugin settings'),
  );
  $form['plugin_settings'] = array(
    '#type' => 'vertical_tabs',
    '#attributes' => array(
      'id' => 'ckeditor-plugin-settings',
    ),
  );

  editor_ckeditor_inject_plugin_settings_form($form, $form_state, $format);

  if (count(element_children($form['plugins'])) === 0) {
    unset($form['plugins']);
    unset($form['plugin_settings_title']);
    unset($form['plugin_settings']);
  }

  // Hidden CKEditor instance. We need a hidden CKEditor instance with all
  // plugins enabled, so we can retrieve CKEditor's per-feature metadata (on
  // which tags, attributes, styles and classes are enabled). This metadata is
  // necessary for certain filters' (for instance, the html_filter filter)
  // settings to be updated accordingly.
  // Get a list of all external plugins and their corresponding files.
  $plugins = editor_ckeditor_plugins();
  $all_external_plugins = array();
  foreach (array_keys($plugins) as $plugin_id) {
    $plugin = $plugins[$plugin_id];
    if (empty($plugin['internal'])) {
      $all_external_plugins[$plugin_id] = $plugin['plugin'];
    }
  }
  // Get a list of all buttons that are provided by all plugins.
  $all_buttons = array_reduce(editor_ckeditor_get_buttons(), function ($result, $item) {
    return array_merge($result, array_keys($item));
  }, array());
  // Build a fake Editor object, which we'll use to generate JavaScript
  // settings for this fake Editor instance.
  $fake_format = clone $format;
  $fake_format->editor = 'ckeditor';
  $fake_format->editor_settings = array(
    // Single toolbar row, single button group, all existing buttons.
    'toolbar' => array(
      0 => array(
        0 => array(
          'name' => 'All existing buttons',
          'items' => $all_buttons,
        ),
      ),
    ),
    'plugins' => $settings['plugins'],
  );

  $config = editor_ckeditor_get_js_settings($fake_format);
  // Remove the ACF configuration that is generated based on filter settings,
  // because otherwise we cannot retrieve per-feature metadata.
  unset($config['allowedContent']);
  $form['hidden_ckeditor'] = array(
    '#markup' => '<div id="ckeditor-hidden" class="hidden"></div>',
    '#attached' => array(
      'js' => array(
        array(
          'type' => 'setting',
          // Group JavaScript settings under one key in order to prevent them
          // from being added to the page multiple times.
          // @see https://www.drupal.org/project/drupal/issues/3397552
          'key' => 'editor_formats',
          'data' => array(
            'ckeditor' => array(
              'hiddenCKEditorConfig' => $config,
            )
          )
        ),
      ),
    ),
  );

  return $form;
}

/**
 * Additional validate handler for filter_admin_format_form().
 */
function editor_ckeditor_settings_form_validate($form, &$form_state) {
}

/**
 * Additional submit handler for filter_admin_format_form().
 */
function editor_ckeditor_settings_form_submit($form, &$form_state) {
  $format = $form_state['format'];

  form_set_value($form['toolbar'], json_decode($form_state['values']['editor']['settings']['toolbar']['button_groups'], TRUE), $form_state);
  unset($form_state['values']['editor']['settings']['toolbar']['button_groups']);

  // Remove the plugin settings' vertical tabs state; no need to save that.
  if (!empty($form_state['values']['editor']['settings']['plugins'])) {
    unset($form_state['values']['editor']['settings']['plugin_settings_title']);
    unset($form_state['values']['editor']['settings']['plugin_settings']);
  }

  // Ensure plugin settings are only saved for plugins that are actually
  // enabled.
  $about_to_be_saved_format_settings = clone $format;
  $about_to_be_saved_format_settings->editor = 'ckeditor';
  $about_to_be_saved_format_settings->editor_settings['toolbar'] = $form_state['values']['editor']['settings']['toolbar'];
  $about_to_be_saved_format_settings->editor_settings['plugins'] = $form_state['values']['editor']['settings']['plugins'];

  $enabled_plugins = _editor_ckeditor_get_enabled_plugins($about_to_be_saved_format_settings);
  $plugin_settings = !empty($form_state['values']['editor']['settings']['plugins']) ? $form_state['values']['editor']['settings']['plugins'] : array();
  foreach (array_keys($plugin_settings) as $plugin_id) {
    if (!in_array($plugin_id, $enabled_plugins, TRUE)) {
      unset($plugin_settings[$plugin_id]);
    }
  }
  $form_state['values']['editor']['settings']['plugins'] = $plugin_settings;

  // Prepare the editor settings and image upload settings for
  // editor_admin_format_editor_submit(). This strips away unwanted form values
  // too, because those never can exist in the already validated Editor config
  // entity.
  $form_state['values']['editor_settings'] = $about_to_be_saved_format_settings->editor_settings;
  $form_state['values']['image_upload'] = $about_to_be_saved_format_settings->image_upload;
}
