<?php

/**
 * @file
 * Editor CKEditor hooks implemented by the Editor CKEditor Test module.
 */

/**
 * Implements hook_editor_ckeditor_plugins().
 */
function editor_ckeditor_test_editor_ckeditor_plugins() {
  // Defines the "Llama" plugin, with a CKEditor "llama" feature.

  // This feature does not correspond to a toolbar button. Because this plugin
  // is not contextual nor includes any buttons, there is no way of actually
  // loading this plugin.
  $plugins['llama'] = array(
    'label' => t('Llama'),
    'internal' => FALSE,
    'plugin' => drupal_get_path('module', 'editor_ckeditor_test') . '/js/llama.js',
  );

  // Defines a "LlamaButton" plugin, with a toolbar builder-enabled "llama" feature.
  $plugins['llama_button'] = array(
    'label' => t('Llama Button'),
    'buttons' => array(
      'Llama' => array(
        'label' => t('Insert Llama'),
      ),
    ),
    'plugin' => drupal_get_path('module', 'editor_ckeditor_test') . '/js/llama_button.js',
  );

  // Defines a "Llama" plugin, with a contextually enabled "llama" feature.
  $plugins['llama_contextual'] = array(
    'label' => t('Language'),
    'contextual' => TRUE,
    'enabled callback' => 'editor_ckeditor_test_llama_contextual_is_enabled',
    'plugin' => drupal_get_path('module', 'editor_ckeditor_test') . '/js/llama_contextual.js',
  );

  // Defines a "LlamaContextualAndButton" plugin, with a contextually OR toolbar
  // builder-enabled "llama" feature.
  $plugins['llama_contextual_and_button'] = array(
    'label' => t('Contextual Llama With Button'),
    'contextual' => TRUE,
    'enabled callback' => 'editor_ckeditor_test_llama_contextual_and_button_is_enabled',
    'buttons' => array(
      'Llama' => array(
        'label' => t('Insert Llama'),
      ),
    ),
    'plugin' => drupal_get_path('module', 'editor_ckeditor_test') . '/js/llama_contextual_and_button.js',
    'settings form' => 'editor_ckeditor_test_llama_contextual_and_button_settings_form',
  );

  // Defines a "LlamaCss" plugin, with an associated "llama" CSS.
  $plugins['llama_css'] = array(
    'label' => t('Llama CSS'),
    'buttons' => array(
      'LlamaCSS' => array(
        'label' => t('Insert Llama CSS'),
      ),
    ),
    'css' => array(drupal_get_path('module', 'editor_ckeditor_test') . '/css/llama.css'),
    'plugin' => drupal_get_path('module', 'editor_ckeditor_test') . '/js/llama_css.js',
  );

  return $plugins;
}

/**
 * Checks if this plugin should be enabled based on the editor configuration.
 *
 * The editor's settings can be retrieved via $format->editor_settings.
 *
 * @param object $format
 *   A text format object.
 *
 * @return bool
 */
function editor_ckeditor_test_llama_contextual_is_enabled($format) {
  // Automatically enable this plugin if the Underline button is enabled.
  foreach ($format->editor_settings['toolbar'] as $row) {
    foreach ($row as $group) {
      if (in_array('Strike', $group['items'])) {
        return TRUE;
      }
    }
  }
  return FALSE;
}

/**
 * Checks if this plugin should be enabled based on the editor configuration.
 *
 * The editor's settings can be retrieved via $format->editor_settings.
 *
 * @param object $format
 *   A text format object.
 *
 * @return bool
 */
function editor_ckeditor_test_llama_contextual_and_button_is_enabled($format) {
  // Automatically enable this plugin if the Underline button is enabled.
  foreach ($format->editor_settings['toolbar'] as $row) {
    foreach ($row as $group) {
      if (in_array('Strike', $group['items'])) {
        return TRUE;
      }
    }
  }
  return FALSE;
}

/**
 * Settings form for the "llama_contextual_and_button" CKEditor plugin.
 */
function editor_ckeditor_test_llama_contextual_and_button_settings_form ($form, &$form_state, $format) {
  // Defaults.
  $config = array('ultra_llama_mode' => FALSE);
  if (isset($format->editor_settings['plugins']['llama_contextual_and_button'])) {
    $config = $format->editor_settings['plugins']['llama_contextual_and_button'];
  }

  $form['ultra_llama_mode'] = array(
    '#title' => t('Ultra llama mode'),
    '#type' => 'checkbox',
    '#default_value' => $config['ultra_llama_mode'],
  );

  return $form;
}
